<?php

namespace SmartCore\Bundle\CMSBundle\Controller;

use Smart\CoreBundle\Controller\Controller;
use SmartCore\Bundle\CMSBundle\Entity\AppearanceHistory;
use Symfony\Component\Finder\Finder;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\HttpFoundation\RedirectResponse;

class AdminAppearanceController extends Controller
{
    /**
     * @return Response
     */
    public function indexAction()
    {
        return $this->render('@CMS/AdminAppearance/index.html.twig', [
            'styles'         => $this->getStyles(),
            'styles_path'    => '/Resources/public/css/',
            'templates'      => $this->getTemplates(),
            'templates_path' => '/Resources/views/',
        ]);
    }

    /**
     * @param Request $request
     * @param string $name
     *
     * @return Response|RedirectResponse
     */
    public function templateEditAction(Request $request, $name)
    {
        $template_file_path = $this->get('kernel')->getBundle('SiteBundle')->getPath().'/Resources/views/'.$name.'.html.twig';

        if (!file_exists($template_file_path)) {
            return $this->redirectToRoute('cms_admin_appearance');
        }

        $template_code = file_get_contents($template_file_path);

        if ($request->isMethod('POST')) {
            $twig = $this->get('twig');

            $template_code = $request->request->get('template_code');

            try {
                $twig->parse($twig->tokenize($template_code));

                $twig->clearCacheFiles();

                $history = new AppearanceHistory();
                $history
                    ->setPath('/Resources/views/')
                    ->setFilename($name.'.html.twig')
                    ->setCode($template_code)
                    ->setUser($this->getUser())
                ;

                $this->persist($history, true);

                file_put_contents($template_file_path, $template_code);

                $this->addFlash('success', 'Шаблон обновлён.');

                return $this->redirectToRoute('cms_admin_appearance_template', ['name' => $name]);
            } catch (\Twig_Error_Syntax $e) {
                $this->addFlash('error', $e->getMessage());
            }
        }

        return $this->render('@CMS/AdminAppearance/template_edit.html.twig', [
            'name'          => $name,
            'templates'     => $this->getTemplates(),
            'template_code' => $template_code,
        ]);
    }

    /**
     * @param string $name
     *
     * @return Response|RedirectResponse
     */
    public function templateHistoryAction($name)
    {
        $em = $this->get('doctrine.orm.entity_manager');

        $histories = $em->getRepository(AppearanceHistory::class)->findBy([
            'path'       => '/Resources/views/',
            'filename'   => $name.'.html.twig',
        ], ['created_at' => 'DESC']);

        return $this->render('@CMS/AdminAppearance/template_history.html.twig', [
            'name' => $name,
            'histories' => $histories,
        ]);
    }

    /**
     * @param string $name
     *
     * @return Response|RedirectResponse
     */
    public function styleHistoryAction($name)
    {
        $em = $this->get('doctrine.orm.entity_manager');

        $histories = $em->getRepository(AppearanceHistory::class)->findBy([
            'path'       => '/Resources/public/css/',
            'filename'   => $name,
        ], ['created_at' => 'DESC']);

        return $this->render('@CMS/AdminAppearance/style_history.html.twig', [
            'name' => $name,
            'histories' => $histories,
        ]);
    }

    /**
     * @param int $id
     *
     * @return Response
     */
    public function historyCodeAction($id)
    {
        $em = $this->get('doctrine.orm.entity_manager');

        $history = $em->getRepository(AppearanceHistory::class)->find($id);

        if (empty($history)) {
            throw $this->createNotFoundException();
        }

        return $this->render('@CMS/AdminAppearance/history_code.html.twig', [
            'history' => $history,
        ]);
    }

    /**
     * @param int $id
     *
     * @return Response
     */
    public function historyRollbackAction($id)
    {
        $em = $this->get('doctrine.orm.entity_manager');

        $history = $em->getRepository(AppearanceHistory::class)->find($id);

        if (empty($history)) {
            throw $this->createNotFoundException();
        }

        $template_file_path = $history->getPath().$history->getFilename();

        if (file_exists($template_file_path)) {
            file_put_contents($template_file_path, $history->getCode());

            $this->get('twig')->clearCacheFiles();
            $this->addFlash('success', 'Файл <b>'.$history->getPath().$history->getFilename().'</b> восстановлен на дату: '.$history->getCreatedAt()->format('Y-m-d H:i:s'));
        } else {
            $this->addFlash('error', 'Файл <b>'.$history->getPath().$history->getFilename().'</b> не найден.');
        }

        return $this->redirectToRoute('cms_admin_appearance');
    }

    /**
     * @param Request $request
     * @param string $name
     *
     * @return Response|RedirectResponse
     */
    public function styleEditAction(Request $request, $name)
    {
        $style_file_path = $this->get('kernel')->getBundle('SiteBundle')->getPath().'/Resources/public/css/'.$name;

        if (!file_exists($style_file_path)) {
            return $this->redirectToRoute('cms_admin_appearance');
        }

        $style_code = file_get_contents($style_file_path);

        if ($request->isMethod('POST')) {
            $style_code = $request->request->get('style_code');

            $history = new AppearanceHistory();
            $history
                ->setPath('/Resources/public/css/')
                ->setFilename($name)
                ->setCode($style_code)
                ->setUser($this->getUser())
            ;

            $this->persist($history, true);

            file_put_contents($style_file_path, $style_code);

            $this->addFlash('success', 'Стиль обновлён.');

            return $this->redirectToRoute('cms_admin_appearance_style', ['name' => $name]);
        }

        return $this->render('CMSBundle:AdminAppearance:style_edit.html.twig', [
            'name'          => $name,
            'styles'        => $this->getStyles(),
            'style_code'    => $style_code,
        ]);
    }

    /**
     * @return array
     */
    protected function getTemplates()
    {
        $finder = new Finder();
        $finder->files()->sortByName()->depth('== 0')->name('*.html.twig')->in($this->get('kernel')->getBundle('SiteBundle')->getPath().'/Resources/views/');

        $templates = [];
        /** @var \Symfony\Component\Finder\SplFileInfo $file */
        foreach ($finder as $file) {
            $templates[] = str_replace('.html.twig', '', $file->getFilename());
        }

        return $templates;
    }

    /**
     * @return array
     */
    protected function getStyles()
    {
        $finder = new Finder();
        $finder->files()->sortByName()->depth('== 0')->name('*.css')->name('*.less')->in($this->get('kernel')->getBundle('SiteBundle')->getPath().'/Resources/public/css/');

        $styles = [];
        /** @var \Symfony\Component\Finder\SplFileInfo $file */
        foreach ($finder as $file) {
            $styles[] = $file->getFilename();
        }

        return $styles;
    }
}
